@@ -33,7 +33,7 @@
 
 
 "mcmc.pois.log" <- 
-  function(data, units.m, meanS, invcov, mcmc.input)
+  function(data, units.m, meanS, invcov, mcmc.input, messages.screen)
 {
 ####
   ## This is the MCMC engine for the spatial Poisson log Normal model ----
@@ -46,26 +46,30 @@
       Htrunc <- mcmc.input$Htrunc
     else Htrunc <- rep(mcmc.input$Htrunc, n)
   }
-  QQ <- t(chol(solve(invcov + diag(data)))) ## needs to be improved
-  sqrtdataQ <- sqrt(data)*QQ              ## needs to be improved
+  QQ <- t(chol(solve(invcov + diag(data))))
+  sqrtdataQ <- sqrt(data)*QQ 
   QtivQ <- diag(n)-crossprod(sqrtdataQ)
   if(any(mcmc.input$S.start=="default")) {
     z <- as.vector(solve(QQ,ifelse(data > 0, log(data), 0) - meanS - log(units.m)))
   }
   else{
-    if(is.numeric(mcmc.input$S.start)){
-      if(length(mcmc.input$S.start) != n) stop("dimension of mcmc-starting-value must equal dimension of data")
-      else z <- as.vector(solve(QQ,mcmc.input$S.start))
+    if(any(mcmc.input$S.start=="random")) z <- rnorm(n)
+    else{
+      if(is.numeric(mcmc.input$S.start)){
+        if(length(mcmc.input$S.start) != n) stop("dimension of mcmc-starting-value must equal dimension of data")
+        else z <- as.vector(solve(QQ,mcmc.input$S.start))
+      }
+      else  stop(" S.start must be a vector of same dimension as data ")
     }
-    else  stop(" S.start must be a vector of same dimension as data ")
   }
   burn.in <- mcmc.input$burn.in
   thin <- mcmc.input$thin
   n.iter <- mcmc.input$n.iter
-  ## ---------------- burn-in ----------------- ######### 
+## ---------------- burn-in ----------------- ######### 
   if(burn.in > 0) {
     mcmc.output <- mcmc.aux(z, data, meanS + log(units.m), QQ, Htrunc, S.scale, 1, burn.in, QtivQ)
-    cat(paste("burn-in = ", burn.in, " is finished. Acc.-rate = ", mcmc.output$acc.rate, "\n"))
+    if(messages.screen) cat(paste("burn-in = ", burn.in, " is finished. Acc.-rate = ", mcmc.output$acc.rate, "\n"))
+    acc.rate.burn.in <- c(burn.in, mcmc.output$acc.rate)
   }
   else mcmc.output <- list(z = z)
 ##### ---------- sampling periode ----------- ###### 
@@ -74,27 +78,26 @@
     n.turn <- 1
   }
   else {
-    if(thin <= 1000){
-      n.temp <- round(1000/thin)
-      n.turn <- round(n.iter/1000)
-    }
-    else{
-      n.temp <- 1
-      n.turn <- round(n.iter/1000)
-    }
+    n.temp <- round(1000/thin)
+    n.turn <- round(n.iter/1000)
   }
-  nsim <- n.turn * n.temp
-  Sdata <- matrix(NA, n, nsim)
+  n.sim <- n.turn * n.temp
+  Sdata <- matrix(NA, n, n.sim)
+  acc.rate <- matrix(NA, n.turn, 2)
   for(i in 1:n.turn) {
     mcmc.output <- mcmc.aux(mcmc.output$z, data, meanS + log(units.m), QQ, Htrunc, S.scale, n.temp, thin, QtivQ)
     Sdata[, (n.temp * (i - 1) + 1):(n.temp * i)] <- mcmc.output$S+meanS
-    cat(paste("iter. numb.", i * n.temp * thin, " : Acc.-rate = ", mcmc.output$acc.rate, "\n"))
+    if(messages.screen) cat(paste("iter. numb.", i * n.temp * thin, " : Acc.-rate = ", mcmc.output$acc.rate, "\n"))
+    acc.rate[i,1] <-  i * n.temp * thin
+    acc.rate[i,2] <- mcmc.output$acc.rate
   }
-  cat(paste("MCMC performed: n.iter. = ", n.iter, "; thinning = ", thin, "; burn.in = ", burn.in, "\n"))
-  if(is.R()) remove("z")
-  else remove(list = c("z"), frame = sys.nframe())
+  if(messages.screen) cat(paste("MCMC performed: n.iter. = ", n.iter, "; thinning = ", thin, "; burn.in = ", burn.in, "\n"))
+  if(burn.in > 0) acc.rate <- as.data.frame(rbind(acc.rate.burn.in,acc.rate))
+  else acc.rate <- as.data.frame(acc.rate)
+  names(acc.rate) <- c("iter.numb", "Acc.rate")
 #########
-  return(Sdata)
+  return(list(Sdata=Sdata, acc.rate=acc.rate))
+
 }
 
 
@@ -132,7 +135,7 @@
 }
 
 "mcmc.pois.boxcox" <- 
-  function(data, units.m, meanS, invcov, mcmc.input, lambda)
+  function(data, units.m, meanS, invcov, mcmc.input, messages.screen, lambda)
 {
 ####
   ## This is the MCMC engine for the spatial Poisson - Normal model with link from the box-cox-family ----
@@ -140,19 +143,22 @@
   n <- length(data)
   S.scale <- mcmc.input$S.scale
   fisher.l <- ifelse(data>0,data^(1-2*lambda)*units.m^(2*lambda),0)
-  QQ <- t(chol(solve(invcov + diag(fisher.l)))) ## needs to be improved 
-  sqrtfiQ <- sqrt(fisher.l)*QQ              ## needs to be improved
+  QQ <- t(chol(solve(invcov + diag(fisher.l)))) 
+  sqrtfiQ <- sqrt(fisher.l)*QQ 
   QtivQ <- diag(n)-crossprod(sqrtfiQ)
   if(any(mcmc.input$S.start=="default")) {
     S <- as.vector(ifelse(data > 0, (data/units.m)^lambda-1, 0)/lambda - meanS )         
     z <- as.vector(solve(QQ,S))
   }
   else{
-    if(is.numeric(mcmc.input$S.start)){
-      if(length(mcmc.input$S.start) != n) stop("dimension of mcmc-starting-value must equal dimension of data")
-      else z <- as.vector(solve(QQ,mcmc.input$S.start))
+    if(any(mcmc.input$S.start=="random")) z <- rnorm(n)
+    else{
+      if(is.numeric(mcmc.input$S.start)){
+        if(length(mcmc.input$S.start) != n) stop("dimension of mcmc-starting-value must equal dimension of data")
+        else z <- as.vector(solve(QQ,mcmc.input$S.start))
+      }
+      else  stop(" S.start must be a vector of same dimension as data ")
     }
-    else  stop(" S.start must be a vector of same dimension as data ")
   }
   if(any(mcmc.input$Htrunc=="default")) Htrunc <- 2*data + 5
   else {
@@ -166,7 +172,8 @@
   ## ---------------- burn-in ----------------- ######### 
   if(burn.in > 0) {
     mcmc.output <- mcmc.boxcox.aux(z, data, units.m, meanS, QQ, Htrunc, S.scale, 1, burn.in, QtivQ, lambda)
-    cat(paste("burn-in = ", burn.in, " is finished. Acc.-rate = ", mcmc.output$acc.rate, "\n"))
+    if(messages.screen) cat(paste("burn-in = ", burn.in, " is finished. Acc.-rate = ", mcmc.output$acc.rate, "\n"))
+    acc.rate.burn.in <- c(burn.in, mcmc.output$acc.rate)
   }
   else mcmc.output <- list(z = z)
 ##### ---------- sampling periode ----------- ###### 
@@ -180,16 +187,22 @@
   }
   n.sim <- n.turn * n.temp
   Sdata <- matrix(NA, n, n.sim)
+  acc.rate <- matrix(NA, n.turn, 2)
   for(i in 1:n.turn) {
     mcmc.output <- mcmc.boxcox.aux(mcmc.output$z, data, units.m, meanS, QQ, Htrunc, S.scale, n.temp, thin, QtivQ, lambda)
     Sdata[, (n.temp * (i - 1) + 1):(n.temp * i)] <- mcmc.output$S+meanS
-    cat(paste("iter. numb.", i * n.temp * thin, " : Acc.-rate = ", mcmc.output$acc.rate, "\n"))
+    if(messages.screen) cat(paste("iter. numb.", i * n.temp * thin, " : Acc.-rate = ", mcmc.output$acc.rate, "\n"))
+    acc.rate[i,1] <-  i * n.temp * thin
+    acc.rate[i,2] <- mcmc.output$acc.rate
   }
-  cat(paste("MCMC performed: n.iter. = ", n.iter, "; thinning = ", thin, "; burn.in = ", burn.in, "\n"))
+  if(messages.screen) cat(paste("MCMC performed: n.iter. = ", n.iter, "; thinning = ", thin, "; burn.in = ", burn.in, "\n"))
+  if(burn.in > 0) acc.rate <- as.data.frame(rbind(acc.rate.burn.in,acc.rate))
+  else acc.rate <- as.data.frame(acc.rate)
+  names(acc.rate) <- c("iter.numb", "Acc.rate")
   if(is.R()) remove("z")
   else remove(list = c("z"), frame = sys.nframe())
 #########
-  return(Sdata)
+  return(list(Sdata=Sdata, acc.rate=acc.rate))
 }
 
 "krige.glm.control" <-
@@ -197,7 +210,7 @@
             nugget, micro.scale, dist.epsilon = 1e-10, aniso.pars, lambda)
 {
   if(type.krige != "ok" & type.krige != "OK" & type.krige != "o.k." & type.krige != "O.K." & type.krige != "sk" & type.krige != "SK" & type.krige != "s.k." & type.krige != "S.K.")
-    stop("pois.log.krige: wrong option in the argument type.krige. It should be \"sk\" or \"ok\"(if ordinary or simple kriging is to be performed)")
+    stop("pois.krige: wrong option in the argument type.krige. It should be \"sk\" or \"ok\"(if ordinary or simple kriging is to be performed)")
   if(type.krige=="OK" | type.krige=="O.K." |type.krige=="o.k.")
     type.krige <- "ok"
   if(type.krige=="SK" | type.krige=="S.K." |type.krige=="s.k.")
@@ -226,25 +239,25 @@
   ##
   if(type.krige == "sk")
     if(is.null(beta) | !is.numeric(beta))
-      stop("\n pois.log.krige: argument beta must be provided in order to perform simple kriging")
+      stop("\n pois.krige: argument beta must be provided in order to perform simple kriging")
   if(micro.scale > nugget)
-    stop("pois.log.krige: micro.scale must be in the interval [0, nugget]")
+    stop("pois.krige: micro.scale must be in the interval [0, nugget]")
   if(!is.null(aniso.pars))
     if(length(aniso.pars) != 2 | !is.numeric(aniso.pars))
-      stop("pois.log.krige: anisotropy parameters must be provided as a numeric vector with two elements: the rotation angle (in radians) and the anisotropy ratio (a number greater than 1)")
+      stop("pois.krige: anisotropy parameters must be provided as a numeric vector with two elements: the rotation angle (in radians) and the anisotropy ratio (a number greater than 1)")
   ##
   if(inherits(trend.d, "formula") | inherits(trend.l, "formula")){
     if(!inherits(trend.d, "formula") | !inherits(trend.l, "formula"))
-      stop("pois.log.krige: trend.d and trend.l must have similar specification")
+      stop("pois.krige: trend.d and trend.l must have similar specification")
   }
   else{
     if((!is.null(class(trend.d)) && class(trend.d)=="trend.spatial") & (!is.null(class(trend.l)) && class(trend.l)=="trend.spatial")){
       if(ncol(trend.d) != ncol(trend.l))
-        stop("pois.log.krige: trend.d and trend.l do not have the same number of columns")
+        stop("pois.krige: trend.d and trend.l do not have the same number of columns")
     }
     else{
       if(trend.d != trend.l)
-        stop("pois.log.krige: trend.l is different from trend.d")
+        stop("pois.krige: trend.l is different from trend.d")
     }
   }
   cov.model <- match.arg(cov.model,
@@ -266,9 +279,8 @@
   return(res)
 }
 
-"pois.log.krige" <- 
-function(geodata, coords = geodata$coords, data = geodata$data, units.m = "default", locations = NULL,
-         mcmc.input, krige, output)
+"pois.krige" <- 
+function(geodata, coords = geodata$coords, data = geodata$data, units.m = "default", locations = NULL, mcmc.input, krige, output)
 {
   if(missing(geodata))
     geodata <- list(coords=coords, data=data)
@@ -282,10 +294,10 @@ function(geodata, coords = geodata$coords, data = geodata$data, units.m = "defau
   else{
     if(is.null(class(krige)) || class(krige) != "krige.geoRglm"){
       if(!is.list(krige))
-        stop("pois.log.krige: the argument krige only takes a list or an output of the function krige.glm.control")
+        stop("pois.krige: the argument krige only takes a list or an output of the function krige.glm.control")
       else{
         krige.names <-c("type.krige","trend.d","trend.l","obj.model","beta","cov.model",
-"cov.pars","kappa","nugget","micro.scale","dist.epsilon","lambda","aniso.pars")
+                        "cov.pars","kappa","nugget","micro.scale","dist.epsilon","lambda","aniso.pars")
         krige.user <- krige
         krige <- list()
         if(length(krige.user) > 0){
@@ -317,7 +329,6 @@ function(geodata, coords = geodata$coords, data = geodata$data, units.m = "defau
   }
   cov.model <- krige$cov.model
   kappa <- krige$kappa
-  if(krige$lambda>0.001) stop("pois.log.krige : lambda > 0 is not implemented ")
   beta <- krige$beta
   cov.pars <- krige$cov.pars
   nugget <- krige$nugget
@@ -326,6 +337,7 @@ function(geodata, coords = geodata$coords, data = geodata$data, units.m = "defau
   trend.d <- krige$trend.d
   trend.l <- krige$trend.l
   dist.epsilon <- krige$dist.epsilon
+  lambda <- krige$lambda
   if(krige$type.krige == "ok") beta.prior <- "flat"
   if(krige$type.krige == "sk") beta.prior <- "deg"
   if(missing(output))
@@ -333,7 +345,7 @@ function(geodata, coords = geodata$coords, data = geodata$data, units.m = "defau
   else{
     if(is.null(class(output)) || class(output) != "output.geoRglm"){
       if(!is.list(output))
-        stop("pois.log.krige: the argument output only takes a list or an output of the function output.glm.control")
+        stop("pois.krige: the argument output only takes a list or an output of the function output.glm.control")
       else{
         output.names <- c("sim.posterior","sim.predict", "keep.mcmc.sim","quantile","threshold","inference","messages.screen")      
         output.user <- output
@@ -380,11 +392,11 @@ function(geodata, coords = geodata$coords, data = geodata$data, units.m = "defau
   ##
   ## preparing for MCMC 
   ##
-  if(missing(mcmc.input)) stop("pois.log.krige: argument mcmc.input must be given")
+  if(missing(mcmc.input)) stop("pois.krige: argument mcmc.input must be given")
   else{
     if(is.null(class(mcmc.input)) || class(mcmc.input) != "mcmc.geoRglm"){
       if(!is.list(mcmc.input))
-        stop("pois.log.krige: the argument mcmc.input only takes a list or an output of the function mcmc.control")
+        stop("pois.krige: the argument mcmc.input only takes a list or an output of the function mcmc.control")
       else{
         mcmc.input.names <- c("S.scale", "Htrunc", "S.start", "burn.in", "thin", "n.iter", "phi.start",  "phi.scale")    
         mcmc.input.user <- mcmc.input
@@ -421,27 +433,32 @@ function(geodata, coords = geodata$coords, data = geodata$data, units.m = "defau
   ##
 ########################----- MCMC ------#####################
   ##
-  if(beta.prior == "deg") {
-    intensity <- exp(mcmc.pois.log(data = data, units.m = units.m, meanS = mean.d, invcov=invcov, mcmc.input = mcmc.input))
-  }
   if(beta.prior == "flat") {
     ivtt <- invcov%*%trend.data    
-    ittivtt <- solve.geoR(crossprod(trend.data, ivtt))    
-    intensity <- exp(mcmc.pois.log(data = data, units.m = units.m, meanS = mean.d,
-                                       invcov=invcov-ivtt%*%ittivtt%*%t(ivtt), mcmc.input = mcmc.input))
+    ittivtt <- solve.geoR(crossprod(trend.data, ivtt))
+    invcov <- invcov-ivtt%*%ittivtt%*%t(ivtt)
+  }
+  if(lambda == 0){ 
+    intensity <- mcmc.pois.log(data = data, units.m = units.m, meanS = mean.d, invcov=invcov, mcmc.input = mcmc.input, messages.screen)
+    acc.rate <- intensity$acc.rate
+    intensity <- exp(intensity$Sdata)
+  }
+  else{
+    intensity <- mcmc.pois.boxcox(data=data, units.m=units.m, meanS=mean.d, invcov=invcov, mcmc.input=mcmc.input, messages.screen, lambda=lambda)  
+    acc.rate <- intensity$acc.rate
+    intensity <- BC.inv(intensity$Sdata, lambda)    
   }
   ##
   ##------------------------------------------------------------
 ######################## ---- prediction ----- #####################
   if(!is.null(locations)) {
     krige <- list(type.krige = krige$type.krige, beta = beta, trend.d = trend.d, trend.l = trend.l, cov.model = cov.model, 
-             cov.pars = cov.pars, kappa = kappa, nugget = nugget, micro.scale = micro.scale, dist.epsilon = dist.epsilon, 
-             aniso.pars = aniso.pars, lambda = 0)
+                  cov.pars = cov.pars, kappa = kappa, nugget = nugget, micro.scale = micro.scale, dist.epsilon = dist.epsilon, 
+                  aniso.pars = aniso.pars, lambda = lambda)
     kpl.result <- krige.conv.extnd(data = intensity, coords = coords, locations = locations, krige = krige,
                                    output = list(n.predictive = ifelse(sim.predict,1,0), signal = TRUE, messages.screen = FALSE))
-    if(is.R()) remove(list = c("intensity"))
-    else remove(list = c("intensity"), frame = sys.nframe())
-    kpl.result$krige.var <- apply(kpl.result$krige.var, 1, mean) + apply(kpl.result$predict, 1, var)
+    remove(list = c("intensity"))
+    kpl.result$krige.var <- apply(kpl.result$krige.var, 1, mean) + apply(kpl.result$predict, 1, var) 
     kpl.result$mcmc.error <- sqrt(asympvar(kpl.result$predict)/ncol(kpl.result$predict))
     kpl.result$predict <- apply(kpl.result$predict, 1, mean)
     if(beta.prior == "flat") {
@@ -452,14 +469,15 @@ function(geodata, coords = geodata$coords, data = geodata$data, units.m = "defau
   }
   else{
     if(beta.prior == "flat") {
-      beta.est <- (ittivtt %*% t(ivtt)) %*% log(intensity)
-      kpl.result <- list(intensity=intensity, beta.est = apply(beta.est, 1, mean))
+      beta.est <- (ittivtt %*% t(ivtt))%*%log(intensity)
+      kpl.result <- list(intensity=intensity, beta.est = apply(beta.est, 1, mean), acc.rate=acc.rate)
     }
-    else kpl.result <- list(intensity=intensity)
+    else kpl.result <- list(intensity=intensity, acc.rate=acc.rate)
   }
   kpl.result$call <- call.fc
 #######################################
   attr(kpl.result, "prediction.locations") <- call.fc$locations
-  class(kpl.result) <- "kriging"
+  ##class(kpl.result) <- "kriging"
+  class(kpl.result) <- "pois.kriging"
   return(kpl.result)
 }
